@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,31 +9,26 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#include "gdboptionspage.h"
 #include <debugger/commonoptionspage.h>
 #include <debugger/debuggeractions.h>
 #include <debugger/debuggercore.h>
 #include <debugger/debuggerinternalconstants.h>
 
+#include <coreplugin/dialogs/ioptionspage.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/variablechooser.h>
 
@@ -46,16 +41,15 @@
 #include <QFormLayout>
 #include <QLabel>
 #include <QLineEdit>
+#include <QPointer>
 #include <QSpinBox>
 #include <QTextEdit>
 
-
 using namespace Core;
 
 namespace Debugger {
 namespace Internal {
 
-
 /////////////////////////////////////////////////////////////////////////
 //
 // GdbOptionsPageWidget - harmless options
@@ -64,13 +58,27 @@ namespace Internal {
 
 class GdbOptionsPageWidget : public QWidget
 {
+    Q_OBJECT
 public:
-    explicit GdbOptionsPageWidget(QWidget *parent = 0);
+    GdbOptionsPageWidget();
     Utils::SavedActionSet group;
 };
 
-GdbOptionsPageWidget::GdbOptionsPageWidget(QWidget *parent)
-    : QWidget(parent)
+class GdbOptionsPage : public Core::IOptionsPage
+{
+    Q_OBJECT
+public:
+    GdbOptionsPage();
+
+    QWidget *widget();
+    void apply();
+    void finish();
+
+private:
+    QPointer<GdbOptionsPageWidget> m_widget;
+};
+
+GdbOptionsPageWidget::GdbOptionsPageWidget()
 {
     auto groupBoxGeneral = new QGroupBox(this);
     groupBoxGeneral->setTitle(GdbOptionsPage::tr("General"));
@@ -297,10 +305,6 @@ GdbOptionsPage::GdbOptionsPage()
     setCategoryIcon(QLatin1String(Constants::DEBUGGER_COMMON_SETTINGS_CATEGORY_ICON));
 }
 
-GdbOptionsPage::~GdbOptionsPage()
-{
-}
-
 QWidget *GdbOptionsPage::widget()
 {
     if (!m_widget)
@@ -330,30 +334,19 @@ void GdbOptionsPage::finish()
 
 class GdbOptionsPageWidget2 : public QWidget
 {
+    Q_OBJECT
 public:
-    explicit GdbOptionsPageWidget2(QWidget *parent = 0);
-
-    QGroupBox *groupBoxDangerous;
-    QLabel *labelDangerous;
-    QCheckBox *checkBoxTargetAsync;
-    QCheckBox *checkBoxAutoEnrichParameters;
-    QCheckBox *checkBoxBreakOnWarning;
-    QCheckBox *checkBoxBreakOnFatal;
-    QCheckBox *checkBoxBreakOnAbort;
-    QCheckBox *checkBoxEnableReverseDebugging;
-    QCheckBox *checkBoxAttemptQuickStart;
-    QCheckBox *checkBoxMultiInferior;
+    GdbOptionsPageWidget2();
 
     Utils::SavedActionSet group;
 };
 
-GdbOptionsPageWidget2::GdbOptionsPageWidget2(QWidget *parent)
-    : QWidget(parent)
+GdbOptionsPageWidget2::GdbOptionsPageWidget2()
 {
-    groupBoxDangerous = new QGroupBox(this);
+    auto groupBoxDangerous = new QGroupBox(this);
     groupBoxDangerous->setTitle(GdbOptionsPage::tr("Extended"));
 
-    labelDangerous = new QLabel(GdbOptionsPage::tr(
+    auto labelDangerous = new QLabel(GdbOptionsPage::tr(
         "The options below should be used with care."));
     labelDangerous->setToolTip(GdbOptionsPage::tr(
         "<html><head/><body>The options below give access to advanced "
@@ -363,11 +356,11 @@ GdbOptionsPageWidget2::GdbOptionsPageWidget2(QWidget *parent)
     f.setItalic(true);
     labelDangerous->setFont(f);
 
-    checkBoxTargetAsync = new QCheckBox(groupBoxDangerous);
+    auto checkBoxTargetAsync = new QCheckBox(groupBoxDangerous);
     checkBoxTargetAsync->setText(GdbOptionsPage::tr(
         "Use asynchronous mode to control the inferior"));
 
-    checkBoxAutoEnrichParameters = new QCheckBox(groupBoxDangerous);
+    auto checkBoxAutoEnrichParameters = new QCheckBox(groupBoxDangerous);
     checkBoxAutoEnrichParameters->setText(GdbOptionsPage::tr(
         "Use common locations for debug information"));
     checkBoxAutoEnrichParameters->setToolTip(GdbOptionsPage::tr(
@@ -375,53 +368,57 @@ GdbOptionsPageWidget2::GdbOptionsPageWidget2(QWidget *parent)
         "of debug information such as <i>/usr/src/debug</i> "
         "when starting GDB.</body></html>"));
 
-    // #fixme: 2.7 Move to common settings page.
-    checkBoxBreakOnWarning = new QCheckBox(groupBoxDangerous);
+    // FIXME: Move to common settings page.
+    auto checkBoxBreakOnWarning = new QCheckBox(groupBoxDangerous);
     checkBoxBreakOnWarning->setText(CommonOptionsPage::msgSetBreakpointAtFunction("qWarning"));
     checkBoxBreakOnWarning->setToolTip(CommonOptionsPage::msgSetBreakpointAtFunctionToolTip("qWarning"));
 
-    checkBoxBreakOnFatal = new QCheckBox(groupBoxDangerous);
+    auto checkBoxBreakOnFatal = new QCheckBox(groupBoxDangerous);
     checkBoxBreakOnFatal->setText(CommonOptionsPage::msgSetBreakpointAtFunction("qFatal"));
     checkBoxBreakOnFatal->setToolTip(CommonOptionsPage::msgSetBreakpointAtFunctionToolTip("qFatal"));
 
-    checkBoxBreakOnAbort = new QCheckBox(groupBoxDangerous);
+    auto checkBoxBreakOnAbort = new QCheckBox(groupBoxDangerous);
     checkBoxBreakOnAbort->setText(CommonOptionsPage::msgSetBreakpointAtFunction("abort"));
     checkBoxBreakOnAbort->setToolTip(CommonOptionsPage::msgSetBreakpointAtFunctionToolTip("abort"));
 
-    checkBoxEnableReverseDebugging = new QCheckBox(groupBoxDangerous);
-    checkBoxEnableReverseDebugging->setText(GdbOptionsPage::tr("Enable reverse debugging"));
-    checkBoxEnableReverseDebugging->setToolTip(GdbOptionsPage::tr(
-        "<html><head/><body><p>Enables stepping backwards.</p><p>"
-        "<b>Note:</b> This feature is very slow and unstable on the GDB side. "
-        "It exhibits unpredictable behavior when going backwards over system "
-        "calls and is very likely to destroy your debugging session.</p></body></html>"));
+    QCheckBox *checkBoxEnableReverseDebugging = 0;
+    if (isReverseDebuggingEnabled()) {
+        checkBoxEnableReverseDebugging = new QCheckBox(groupBoxDangerous);
+        checkBoxEnableReverseDebugging->setText(GdbOptionsPage::tr("Enable reverse debugging"));
+        checkBoxEnableReverseDebugging->setToolTip(GdbOptionsPage::tr(
+           "<html><head/><body><p>Enables stepping backwards.</p><p>"
+           "<b>Note:</b> This feature is very slow and unstable on the GDB side. "
+           "It exhibits unpredictable behavior when going backwards over system "
+           "calls and is very likely to destroy your debugging session.</p></body></html>"));
+    }
 
-    checkBoxAttemptQuickStart = new QCheckBox(groupBoxDangerous);
+    auto checkBoxAttemptQuickStart = new QCheckBox(groupBoxDangerous);
     checkBoxAttemptQuickStart->setText(GdbOptionsPage::tr("Attempt quick start"));
     checkBoxAttemptQuickStart->setToolTip(GdbOptionsPage::tr(
         "<html><head/><body>Postpones reading debug information as long as possible. "
         "This can result in faster startup times at the price of not being able to "
         "set breakpoints by file and number.</body></html>"));
 
-    checkBoxMultiInferior = new QCheckBox(groupBoxDangerous);
+    auto checkBoxMultiInferior = new QCheckBox(groupBoxDangerous);
     checkBoxMultiInferior->setText(GdbOptionsPage::tr("Debug all children"));
     checkBoxMultiInferior->setToolTip(GdbOptionsPage::tr(
         "<html><head/><body>Keeps debugging all children after a fork."
         "</body></html>"));
 
 
-    QFormLayout *formLayout = new QFormLayout(groupBoxDangerous);
+    auto formLayout = new QFormLayout(groupBoxDangerous);
     formLayout->addRow(labelDangerous);
     formLayout->addRow(checkBoxTargetAsync);
     formLayout->addRow(checkBoxAutoEnrichParameters);
     formLayout->addRow(checkBoxBreakOnWarning);
     formLayout->addRow(checkBoxBreakOnFatal);
     formLayout->addRow(checkBoxBreakOnAbort);
-    formLayout->addRow(checkBoxEnableReverseDebugging);
+    if (checkBoxEnableReverseDebugging)
+        formLayout->addRow(checkBoxEnableReverseDebugging);
     formLayout->addRow(checkBoxAttemptQuickStart);
     formLayout->addRow(checkBoxMultiInferior);
 
-    QGridLayout *gridLayout = new QGridLayout(this);
+    auto gridLayout = new QGridLayout(this);
     gridLayout->addWidget(groupBoxDangerous, 0, 0, 2, 1);
 
     group.insert(action(AutoEnrichParameters), checkBoxAutoEnrichParameters);
@@ -431,9 +428,25 @@ GdbOptionsPageWidget2::GdbOptionsPageWidget2(QWidget *parent)
     group.insert(action(BreakOnAbort), checkBoxBreakOnAbort);
     group.insert(action(AttemptQuickStart), checkBoxAttemptQuickStart);
     group.insert(action(MultiInferior), checkBoxMultiInferior);
-    group.insert(action(EnableReverseDebugging), checkBoxEnableReverseDebugging);
+    if (checkBoxEnableReverseDebugging)
+        group.insert(action(EnableReverseDebugging), checkBoxEnableReverseDebugging);
 }
 
+// The "Dangerous" options.
+class GdbOptionsPage2 : public Core::IOptionsPage
+{
+    Q_OBJECT
+public:
+    GdbOptionsPage2();
+
+    QWidget *widget();
+    void apply();
+    void finish();
+
+private:
+    QPointer<GdbOptionsPageWidget2> m_widget;
+};
+
 GdbOptionsPage2::GdbOptionsPage2()
 {
     setId("M.Gdb2");
@@ -443,10 +456,6 @@ GdbOptionsPage2::GdbOptionsPage2()
     setCategoryIcon(QLatin1String(Constants::DEBUGGER_COMMON_SETTINGS_CATEGORY_ICON));
 }
 
-GdbOptionsPage2::~GdbOptionsPage2()
-{
-}
-
 QWidget *GdbOptionsPage2::widget()
 {
     if (!m_widget)
@@ -468,5 +477,15 @@ void GdbOptionsPage2::finish()
     }
 }
 
+// Registration
+
+void addGdbOptionPages(QList<IOptionsPage *> *opts)
+{
+    opts->push_back(new GdbOptionsPage);
+    opts->push_back(new GdbOptionsPage2);
+}
+
 } // namespace Internal
 } // namespace Debugger
+
+#include "gdboptionspage.moc"
