@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -42,7 +37,6 @@
 
 QT_BEGIN_NAMESPACE
 class QAction;
-class QComboBox;
 class QMenu;
 class QToolButton;
 class QUrl;
@@ -86,50 +80,31 @@ public:
     static HelpViewer *createHelpViewer(qreal zoom);
 
 private slots:
-    void unregisterOldQtCreatorDocumentation();
-
     void modeChanged(Core::IMode *mode, Core::IMode *old);
 
     void showContextHelp();
     void activateIndex();
     void activateContents();
-    void activateSearch();
-    void activateOpenPages();
-    void activateBookmarks();
-
-    void addBookmark();
-    void updateFilterComboBox();
-    void filterDocumentation(const QString &customFilter);
 
     void saveExternalWindowSettings();
-    void switchToHelpMode(const QUrl &source);
+    void showLinkInHelpMode(const QUrl &source);
+    void showLinksInHelpMode(const QMap<QString, QUrl> &links, const QString &key);
     void slotHideRightPane();
-    void setSideBarVisible(bool visible);
 
     void updateSideBarSource();
     void updateSideBarSource(const QUrl &newUrl);
 
-    void fontChanged();
-
-    void updateCloseButton();
     void setupHelpEngineIfNeeded();
 
     void highlightSearchTermsInContextHelp();
     void handleHelpRequest(const QUrl &url, Core::HelpManager::HelpViewerLocation location);
 
-    void slotAboutToShowBackMenu();
-    void slotAboutToShowNextMenu();
     void slotOpenSupportPage();
     void slotReportBug();
 
-    void onSideBarVisibilityChanged();
-
 private:
-    void setupUi();
     void resetFilter();
     void activateHelpMode();
-    Utils::StyledBar *createWidgetToolBar();
-    Utils::StyledBar *createIconToolBar(bool external);
     bool canShowHelpSideBySide() const;
     HelpViewer *viewerForHelpViewerLocation(Core::HelpManager::HelpViewerLocation location);
     HelpViewer *viewerForContextHelp();
@@ -138,47 +113,24 @@ private:
     HelpViewer *externalHelpViewer();
 
     void doSetupIfNeeded();
-    Core::HelpManager::HelpViewerLocation contextHelpOption() const;
-    void setupNavigationMenus(QAction *back, QAction *next, QWidget *parent);
 
 private:
     HelpMode *m_mode;
     CentralWidget *m_centralWidget;
     HelpWidget *m_rightPaneSideBarWidget;
 
-    Core::SideBarItem *m_contentItem;
-    Core::SideBarItem *m_indexItem;
-    Core::SideBarItem *m_searchItem;
-    Core::SideBarItem *m_bookmarkItem;
-    Core::SideBarItem *m_openPagesItem;
-
     DocSettingsPage *m_docSettingsPage;
     FilterSettingsPage *m_filterSettingsPage;
-    GeneralSettingsPage *m_generalSettingsPage;
     SearchTaskHandler *m_searchTaskHandler;
 
-    QComboBox *m_filterComboBox;
-    Core::SideBar *m_sideBar;
-
-    bool m_firstModeChange;
+    bool m_setupNeeded;
     LocalHelpManager *m_helpManager;
     OpenPagesManager *m_openPagesManager;
-    Core::MiniSplitter *m_splitter;
-
-    QToolButton *m_closeButton;
 
     QString m_contextHelpHighlightId;
 
     QPointer<HelpWidget> m_externalWindow;
     QRect m_externalWindowState;
-
-    QMenu *m_backMenu;
-    QMenu *m_nextMenu;
-    Utils::StyledBar *m_internalHelpBar;
-    Utils::StyledBar *m_externalHelpBar;
-
-    bool m_isSidebarVisible;
-    QAction *m_toggleSideBarAction;
 };
 
 } // namespace Internal
